import os
from PyQt5 import uic
from PyQt5.QtWidgets import QMainWindow
from pyqtgraph.graphicsItems.PlotItem import PlotItem
from pyqtgraph.imageview import ImageView
from pyqtgraph import ColorMap


class mainWindow(QMainWindow):
    def __init__(self):
        super(mainWindow, self).__init__()
        self.load_ui()
        self.mainWindow_setup()


    def load_ui(self):
        path = os.path.join(os.path.dirname(__file__), "form.ui")
        uic.loadUi(path, self)

    def mainWindow_setup(self):
         self.setWindowTitle("ploting the crosssections")


class mainWindow2(QMainWindow):
    def __init__(self):
        super(mainWindow2, self).__init__()
        self.load_ui()
        self.mainWindow_setup()


    def load_ui(self):
        path = os.path.join(os.path.dirname(__file__), "second_app.ui")
        uic.loadUi(path, self)

    def mainWindow_setup(self):
         self.setWindowTitle("ploting the crosssections")
         self.graphicsView_5.getView().invertY(False)
         self.graphicsView_6.getView().invertY(False)
         self.graphicsView_7.getView().invertY(False)
         self.graphicsView_8.getView().invertY(False)



# same as ImageView but with coordinates
class myImageView(ImageView):
    def __init__(self, *args, **kwargs):
        super().__init__(view=PlotItem(), *args, **kwargs)
        map = ColorMap(pos=[0, 0.5, 1], color=[(0,0,255), (255,255,255), (255,0,0)])
        self.setColorMap(map)
        # self.setPredefinedGradient('flame')


from PyQt5.QtWidgets import QApplication

from .controllers import MainController, MainController2
from .views import mainWindow, mainWindow2

class App(QApplication):
    def __init__(self, sys_argv):
        super(App, self).__init__(sys_argv)
        self.widget = mainWindow()
        self.widget.show()
        self.con = MainController(self.widget)


class App2(QApplication):
    def __init__(self, sys_argv):
        super(App2, self).__init__(sys_argv)
        self.widget = mainWindow2()
        self.widget.show()
        self.con = MainController2(self.widget)


import numpy as np

class ImgData:
    # def __init__(self):
    #     pass

    def loadData(self,filename):
        self.raw_data = np.loadtxt(filename)

        self.x = sorted(set(self.raw_data[:,0]))
        self.y = sorted(set(self.raw_data[:,1]))
        self.z = sorted(set(self.raw_data[:,2]))

        x_mapping = { elem:i for i, elem in enumerate(self.x)}
        y_mapping = { elem:i for i, elem in enumerate(self.y)}
        z_mapping = { elem:i for i, elem in enumerate(self.z)}

        self.number_of_params = self.raw_data.shape[1]-3
        self.all_params = []
        for i in range(self.number_of_params):
            tmp = np.empty( (len(self.x), len(self.y), len(self.z)) )
            self.all_params.append(tmp)

        for row in self.raw_data:
            x_ind = x_mapping[row[0]]
            y_ind = y_mapping[row[1]]
            z_ind = z_mapping[row[2]]

            for i in range(self.number_of_params):
                param_index = i + 3
                self.all_params[i][x_ind,y_ind,z_ind] = row[param_index]


        self.img = self.all_params[0]

    def change_param_number(self, param_number):
        self.img = self.all_params[param_number]

    def loadFakeData(self):
        self.x = np.linspace(1,10,100)
        self.y = np.linspace(50,70,500)
        self.z = np.linspace(80,150,250)
        
        # raw data from GG
        self.img = np.random.rand(len(self.x), len(self.y), len(self.z))
        self.img[0,0,0] = np.nan

    
    #xy slice
    def get_slice_parallel(self):
        scale = self.x[1]-self.x[0], self.y[1]-self.y[0]
        pos = self.x[0] - scale[0]/2, self.y[0] - scale[1]/2
        return pos, scale, np.transpose(self.img, (2,0,1))


    # xz slice
    def get_slice_perp(self):
        scale = self.x[1]-self.x[0], self.z[1]-self.z[0]
        pos = self.x[0] - scale[0]/2, self.z[0] - scale[1]/2
        return pos, scale, np.transpose(self.img, (1,0,2))


    #yz slice
    def get_slice_perp2(self):
        scale = self.y[1]-self.y[0], self.z[1]-self.z[0]
        pos = self.y[0] - scale[0]/2, self.z[0] - scale[1]/2
        return pos, scale, np.transpose(self.img, (0,1,2))


from PyQt5.QtWidgets import QFileDialog, QLabel
import numpy as np
import os
from .models import ImgData


class MainController():
    def __init__(self, widget):
        self.widget = widget
        self._create_connections()
        self.data = ImgData()

    
    def _create_connections(self):
        #buttons
        self.widget.pushButton.clicked.connect(self.loadData)

        #connect slider in windows to update
        self.widget.graphicsView.timeLine.sigPositionChanged.connect(self.updateTextWithSliders)
        self.widget.graphicsView_2.timeLine.sigPositionChanged.connect(self.updateTextWithSliders)
        self.widget.graphicsView_3.timeLine.sigPositionChanged.connect(self.updateTextWithSliders)
        self.widget.actionLoad_Test_Data.triggered.connect(self.loadFakeData)
        self.widget.spinBox.valueChanged.connect(self.paramValueChanged)


    #slots
    def loadData(self):
        filename = QFileDialog.getOpenFileName()
        self.data.loadData(filename[0])
        #rename the choose file dialog
        self.widget.lineEdit_4.setText(filename[0])
        self.plotData()
        self.widget.spinBox.setMaximum(self.data.number_of_params)

        #update what is slider value
        self.updateTextWithSliders()


    def loadFakeData(self):
        self.data.loadFakeData()
        self.plotData()
        self.updateTextWithSliders()

    def paramValueChanged(self):
        param_num = self.widget.spinBox.value()
        self.data.change_param_number(param_num-1)
        
        z_ind = self.widget.graphicsView.currentIndex
        y_ind = self.widget.graphicsView_2.currentIndex
        x_ind = self.widget.graphicsView_3.currentIndex

        self.plotData()

        self.widget.graphicsView.setCurrentIndex(z_ind)
        self.widget.graphicsView_2.setCurrentIndex(y_ind)
        self.widget.graphicsView_3.setCurrentIndex(x_ind)


        self.updateTextWithSliders()


    def plotData(self):
        posSlice, scaleSlice, imgSlice = self.data.get_slice_parallel()
        self.widget.graphicsView.setImage(imgSlice, pos=posSlice, scale=scaleSlice, autoRange=True, autoHistogramRange=True)

        posPerp, scalePerp, imgPerp = self.data.get_slice_perp()
        self.widget.graphicsView_2.setImage(imgPerp, pos=posPerp, scale=scalePerp, autoRange=True, autoHistogramRange=True)

        posPerp2, scalePerp2, imgPerp2 = self.data.get_slice_perp2()
        self.widget.graphicsView_3.setImage(imgPerp2, pos=posPerp2, scale=scalePerp2, autoRange=True, autoHistogramRange=True)

    def updateTextWithSliders(self):
        z_ind = self.widget.graphicsView.currentIndex
        y_ind = self.widget.graphicsView_2.currentIndex
        x_ind = self.widget.graphicsView_3.currentIndex

        self.widget.lineEdit.setText(f"z={self.data.z[z_ind]}")
        self.widget.lineEdit_3.setText(f"y={self.data.y[y_ind]}")
        self.widget.lineEdit_2.setText(f"x={self.data.x[x_ind]}")


class MainController2():
    def __init__(self, widget):
        self.widget = widget
        self._create_connections()
        self.data = ImgData()

    
    def _create_connections(self):
        #buttons
        self.widget.pushButton.clicked.connect(self.loadData)

        #connect slider in windows to update
        self.widget.graphicsView.sigTimeChanged.connect(self.updateTextWithSliders)
        self.widget.graphicsView_2.sigTimeChanged.connect(self.updateTextWithSliders)
        self.widget.graphicsView_3.sigTimeChanged.connect(self.updateTextWithSliders)
        self.widget.graphicsView_4.sigTimeChanged.connect(self.updateTextWithSliders)
        self.widget.graphicsView_5.sigTimeChanged.connect(self.updateTextWithSliders2)
        self.widget.graphicsView_6.sigTimeChanged.connect(self.updateTextWithSliders2)
        self.widget.graphicsView_7.sigTimeChanged.connect(self.updateTextWithSliders2)
        self.widget.graphicsView_8.sigTimeChanged.connect(self.updateTextWithSliders2)


    #slots
    def loadData(self):
        filename = QFileDialog.getOpenFileName()
        self.data.loadData(filename[0])
        #rename the choose file dialog
        self.widget.lineEdit_4.setText(filename[0])
        self.plotData()

        #update what is slider value
        self.updateTextWithSliders(0)
        self.updateTextWithSliders2(0)


    def plotData(self):
        self.data.change_param_number(0)
        posSlice, scaleSlice, imgSlice = self.data.get_slice_parallel()
        self.widget.graphicsView.setImage(imgSlice, pos=posSlice, scale=scaleSlice, autoRange=True, autoHistogramRange=True)

        self.data.change_param_number(1)
        posSlice, scaleSlice, imgSlice = self.data.get_slice_parallel()
        self.widget.graphicsView_2.setImage(imgSlice, pos=posSlice, scale=scaleSlice, autoRange=True, autoHistogramRange=True)

        self.data.change_param_number(2)
        posSlice, scaleSlice, imgSlice = self.data.get_slice_parallel()
        self.widget.graphicsView_3.setImage(imgSlice, pos=posSlice, scale=scaleSlice, autoRange=True, autoHistogramRange=True)

        self.data.change_param_number(3)
        posSlice, scaleSlice, imgSlice = self.data.get_slice_parallel()
        self.widget.graphicsView_4.setImage(imgSlice, pos=posSlice, scale=scaleSlice, autoRange=True, autoHistogramRange=True)


        self.data.change_param_number(0)
        posSlice, scaleSlice, imgSlice = self.data.get_slice_perp()
        self.widget.graphicsView_5.setImage(imgSlice, pos=posSlice, scale=scaleSlice, autoRange=True, autoHistogramRange=True)

        self.data.change_param_number(1)
        posSlice, scaleSlice, imgSlice = self.data.get_slice_perp()
        self.widget.graphicsView_6.setImage(imgSlice, pos=posSlice, scale=scaleSlice, autoRange=True, autoHistogramRange=True)

        self.data.change_param_number(2)
        posSlice, scaleSlice, imgSlice = self.data.get_slice_perp()
        self.widget.graphicsView_7.setImage(imgSlice, pos=posSlice, scale=scaleSlice, autoRange=True, autoHistogramRange=True)

        self.data.change_param_number(3)
        posSlice, scaleSlice, imgSlice = self.data.get_slice_perp()
        self.widget.graphicsView_8.setImage(imgSlice, pos=posSlice, scale=scaleSlice, autoRange=True, autoHistogramRange=True)



    def updateTextWithSliders(self, ind):
        self.widget.graphicsView.setCurrentIndex(ind)
        self.widget.graphicsView_2.setCurrentIndex(ind)
        self.widget.graphicsView_3.setCurrentIndex(ind)
        self.widget.graphicsView_4.setCurrentIndex(ind)

        self.widget.lineEdit.setText(f"z={self.data.z[ind]}")
        self.widget.lineEdit_3.setText(f"z={self.data.z[ind]}")
        self.widget.lineEdit_2.setText(f"z={self.data.z[ind]}")
        self.widget.lineEdit_5.setText(f"z={self.data.z[ind]}")

    def updateTextWithSliders2(self, ind):
        self.widget.graphicsView_5.setCurrentIndex(ind)
        self.widget.graphicsView_6.setCurrentIndex(ind)
        self.widget.graphicsView_7.setCurrentIndex(ind)
        self.widget.graphicsView_8.setCurrentIndex(ind)

        self.widget.lineEdit_6.setText(f"y={self.data.z[ind]}")
        self.widget.lineEdit_7.setText(f"y={self.data.z[ind]}")
        self.widget.lineEdit_8.setText(f"y={self.data.z[ind]}")
        self.widget.lineEdit_9.setText(f"y={self.data.z[ind]}")





import pathlib
from setuptools import setup, find_packages

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# This call to setup() does all the work
setup(
    name="slaven92-QTvisualiser",
    version="1.1.1",
    description="Visualise cross section of a 3D image",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/slaven92/QTvisualiser",
    author="Slaven Tepsic",
    author_email="slaven.tepsic@icfo.eu",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
    ],
    packages=["QTvisualiser"],
    include_package_data=True,
    install_requires=["PyQt5", "pyqtgraph"],
    entry_points={
        "console_scripts": [
            "visualise=QTvisualiser.__main__:main",
        ]
    },
)
